"""Initial database structure.

Revision ID: bb9b50b24dfa
Revises: 
Create Date: 2024-08-01 17:55:01.941354

"""
from typing import Sequence, Union

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

# revision identifiers, used by Alembic.
revision: str = 'bb9b50b24dfa'
down_revision: Union[str, None] = None
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    sa.Enum('system', 'admin', 'moderator', 'authorized', 'unauthorized', name='approletype', schema='quackamollie_types').create(op.get_bind())
    sa.Enum('private', 'group', 'supergroup', 'local', name='chattype', schema='quackamollie_types').create(op.get_bind())
    sa.Enum('system', 'user', name='usertype', schema='quackamollie_types').create(op.get_bind())
    op.create_table('app_roles',
    sa.Column('id', sa.Integer(), autoincrement=True, nullable=False),
    sa.Column('role_type', postgresql.ENUM('system', 'admin', 'moderator', 'authorized', 'unauthorized', name='approletype', schema='quackamollie_types', create_type=False), nullable=True),
    sa.Column('created_at', sa.DateTime(), nullable=True),
    sa.Column('updated_at', sa.DateTime(), nullable=True),
    sa.PrimaryKeyConstraint('id', name=op.f('pk_app_roles')),
    schema='quackamollie'
    )
    op.create_table('chats',
    sa.Column('id', sa.BigInteger(), nullable=False),
    sa.Column('chat_name', sa.String(length=80), nullable=False),
    sa.Column('chat_type', postgresql.ENUM('private', 'group', 'supergroup', 'local', name='chattype', schema='quackamollie_types', create_type=False), nullable=True),
    sa.Column('created_at', sa.DateTime(), nullable=True),
    sa.Column('updated_at', sa.DateTime(), nullable=True),
    sa.PrimaryKeyConstraint('id', name=op.f('pk_chats')),
    schema='quackamollie'
    )
    op.create_table('model_manager_types',
    sa.Column('id', sa.Integer(), autoincrement=True, nullable=False),
    sa.Column('model_manager_class', sa.String(length=320), nullable=False),
    sa.PrimaryKeyConstraint('id', name=op.f('pk_model_manager_types')),
    schema='quackamollie_types'
    )
    op.create_table('app_permissions',
    sa.Column('id', sa.Integer(), autoincrement=True, nullable=False),
    sa.Column('role_id', sa.Integer(), nullable=False),
    sa.Column('name', sa.String(length=80), nullable=False),
    sa.Column('type', sa.String(length=80), nullable=False),
    sa.Column('created_at', sa.DateTime(), nullable=True),
    sa.Column('updated_at', sa.DateTime(), nullable=True),
    sa.ForeignKeyConstraint(['role_id'], ['quackamollie.app_roles.id'], name=op.f('fk_app_permissions_role_id_app_roles')),
    sa.PrimaryKeyConstraint('id', name=op.f('pk_app_permissions')),
    schema='quackamollie'
    )
    op.create_index(op.f('ix_quackamollie_app_permissions_role_id'), 'app_permissions', ['role_id'], unique=False, schema='quackamollie')
    op.create_table('model_configs',
    sa.Column('id', sa.Integer(), autoincrement=True, nullable=False),
    sa.Column('config_name', sa.String(length=80), nullable=False),
    sa.Column('model_manager_type_id', sa.Integer(), nullable=False),
    sa.Column('model_name', sa.String(length=320), nullable=False),
    sa.Column('config', sa.String(length=1024), nullable=True),
    sa.Column('created_at', sa.DateTime(), nullable=True),
    sa.Column('updated_at', sa.DateTime(), nullable=True),
    sa.ForeignKeyConstraint(['model_manager_type_id'], ['quackamollie_types.model_manager_types.id'], name=op.f('fk_model_configs_model_manager_type_id_model_manager_types')),
    sa.PrimaryKeyConstraint('id', name=op.f('pk_model_configs')),
    schema='quackamollie'
    )
    op.create_table('users',
    sa.Column('id', sa.BigInteger(), nullable=False),
    sa.Column('user_type', postgresql.ENUM('system', 'user', name='usertype', schema='quackamollie_types', create_type=False), nullable=True),
    sa.Column('app_role_id', sa.Integer(), nullable=False),
    sa.Column('full_name', sa.String(length=80), nullable=False),
    sa.Column('username', sa.String(length=80), nullable=True),
    sa.Column('first_name', sa.String(length=80), nullable=True),
    sa.Column('last_name', sa.String(length=80), nullable=True),
    sa.Column('created_at', sa.DateTime(), nullable=True),
    sa.Column('updated_at', sa.DateTime(), nullable=True),
    sa.ForeignKeyConstraint(['app_role_id'], ['quackamollie.app_roles.id'], name=op.f('fk_users_app_role_id_app_roles')),
    sa.PrimaryKeyConstraint('id', name=op.f('pk_users')),
    schema='quackamollie'
    )
    op.create_index(op.f('ix_quackamollie_users_app_role_id'), 'users', ['app_role_id'], unique=False, schema='quackamollie')
    op.create_table('chat_members',
    sa.Column('user_id', sa.BigInteger(), nullable=False),
    sa.Column('chat_id', sa.BigInteger(), nullable=False),
    sa.Column('created_at', sa.DateTime(), nullable=True),
    sa.Column('updated_at', sa.DateTime(), nullable=True),
    sa.ForeignKeyConstraint(['chat_id'], ['quackamollie.chats.id'], name=op.f('fk_chat_members_chat_id_chats')),
    sa.ForeignKeyConstraint(['user_id'], ['quackamollie.users.id'], name=op.f('fk_chat_members_user_id_users')),
    sa.PrimaryKeyConstraint('user_id', 'chat_id', name=op.f('pk_chat_members')),
    schema='quackamollie'
    )
    op.create_table('chat_messages',
    sa.Column('id', sa.BigInteger(), nullable=False),
    sa.Column('user_id', sa.BigInteger(), nullable=False),
    sa.Column('chat_id', sa.BigInteger(), nullable=False),
    sa.Column('content', sa.String(length=4096), nullable=False),
    sa.Column('sent_at_datetime', sa.DateTime(), nullable=False),
    sa.Column('active', sa.Boolean(), nullable=False),
    sa.Column('created_at', sa.DateTime(), nullable=True),
    sa.Column('updated_at', sa.DateTime(), nullable=True),
    sa.ForeignKeyConstraint(['chat_id'], ['quackamollie.chats.id'], name=op.f('fk_chat_messages_chat_id_chats'), ondelete='CASCADE'),
    sa.ForeignKeyConstraint(['user_id'], ['quackamollie.users.id'], name=op.f('fk_chat_messages_user_id_users')),
    sa.PrimaryKeyConstraint('id', name=op.f('pk_chat_messages')),
    schema='quackamollie'
    )
    op.create_index(op.f('ix_quackamollie_chat_messages_chat_id'), 'chat_messages', ['chat_id'], unique=False, schema='quackamollie')
    op.create_index(op.f('ix_quackamollie_chat_messages_sent_at_datetime'), 'chat_messages', ['sent_at_datetime'], unique=False, schema='quackamollie')
    op.create_index(op.f('ix_quackamollie_chat_messages_user_id'), 'chat_messages', ['user_id'], unique=False, schema='quackamollie')
    op.create_table('chat_settings',
    sa.Column('id', sa.Integer(), autoincrement=True, nullable=False),
    sa.Column('chat_id', sa.BigInteger(), nullable=False),
    sa.Column('chat_type_override', postgresql.ENUM('private', 'group', 'supergroup', 'local', name='chattype', schema='quackamollie_types', create_type=False), nullable=True),
    sa.Column('model_config_id', sa.Integer(), nullable=True),
    sa.Column('created_at', sa.DateTime(), nullable=True),
    sa.Column('updated_at', sa.DateTime(), nullable=True),
    sa.ForeignKeyConstraint(['chat_id'], ['quackamollie.chats.id'], name=op.f('fk_chat_settings_chat_id_chats'), ondelete='CASCADE'),
    sa.ForeignKeyConstraint(['model_config_id'], ['quackamollie.model_configs.id'], name=op.f('fk_chat_settings_model_config_id_model_configs')),
    sa.PrimaryKeyConstraint('id', name=op.f('pk_chat_settings')),
    schema='quackamollie'
    )
    op.create_index(op.f('ix_quackamollie_chat_settings_chat_id'), 'chat_settings', ['chat_id'], unique=True, schema='quackamollie')
    op.create_index(op.f('ix_quackamollie_chat_settings_model_config_id'), 'chat_settings', ['model_config_id'], unique=False, schema='quackamollie')
    op.create_table('model_config_permissions',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('model_config_id', sa.Integer(), nullable=False),
    sa.Column('read', sa.Boolean(), nullable=False),
    sa.Column('write', sa.Boolean(), nullable=False),
    sa.Column('delete', sa.Boolean(), nullable=False),
    sa.Column('hidden', sa.Boolean(), nullable=False),
    sa.ForeignKeyConstraint(['id'], ['quackamollie.app_permissions.id'], name=op.f('fk_model_config_permissions_id_app_permissions'), ondelete='CASCADE'),
    sa.ForeignKeyConstraint(['model_config_id'], ['quackamollie.model_configs.id'], name=op.f('fk_model_config_permissions_model_config_id_model_configs')),
    sa.PrimaryKeyConstraint('id', name=op.f('pk_model_config_permissions')),
    schema='quackamollie'
    )
    op.create_index(op.f('ix_quackamollie_model_config_permissions_model_config_id'), 'model_config_permissions', ['model_config_id'], unique=False, schema='quackamollie')
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index(op.f('ix_quackamollie_model_config_permissions_model_config_id'), table_name='model_config_permissions', schema='quackamollie')
    op.drop_table('model_config_permissions', schema='quackamollie')
    op.drop_index(op.f('ix_quackamollie_chat_settings_model_config_id'), table_name='chat_settings', schema='quackamollie')
    op.drop_index(op.f('ix_quackamollie_chat_settings_chat_id'), table_name='chat_settings', schema='quackamollie')
    op.drop_table('chat_settings', schema='quackamollie')
    op.drop_index(op.f('ix_quackamollie_chat_messages_user_id'), table_name='chat_messages', schema='quackamollie')
    op.drop_index(op.f('ix_quackamollie_chat_messages_sent_at_datetime'), table_name='chat_messages', schema='quackamollie')
    op.drop_index(op.f('ix_quackamollie_chat_messages_chat_id'), table_name='chat_messages', schema='quackamollie')
    op.drop_table('chat_messages', schema='quackamollie')
    op.drop_table('chat_members', schema='quackamollie')
    op.drop_index(op.f('ix_quackamollie_users_app_role_id'), table_name='users', schema='quackamollie')
    op.drop_table('users', schema='quackamollie')
    op.drop_table('model_configs', schema='quackamollie')
    op.drop_index(op.f('ix_quackamollie_app_permissions_role_id'), table_name='app_permissions', schema='quackamollie')
    op.drop_table('app_permissions', schema='quackamollie')
    op.drop_table('model_manager_types', schema='quackamollie_types')
    op.drop_table('chats', schema='quackamollie')
    op.drop_table('app_roles', schema='quackamollie')
    sa.Enum('system', 'user', name='usertype', schema='quackamollie_types').drop(op.get_bind())
    sa.Enum('private', 'group', 'supergroup', 'local', name='chattype', schema='quackamollie_types').drop(op.get_bind())
    sa.Enum('system', 'admin', 'moderator', 'authorized', 'unauthorized', name='approletype', schema='quackamollie_types').drop(op.get_bind())
    # ### end Alembic commands ###
